function [] = Demo2D_v2(imname)

%% Initialization
try
    matlabpool(6)
end

close all
clc

%% Initialize the problem variables
thres = 0;%10^2;
blocksize = 16;
TotalCoef = 0;
NLoop = 1;

K = 0.3; 
%% Read the image
I = imread(imname);
I = double(I);
NoBlock = (size(I,2)/blocksize).^2;
I_org = I;
N = size(I,1);

%% Prepare the function statements
fun = @(block_struct) dctfilter(block_struct.data,thres);
fun2 = @(block_struct) idct2(block_struct.data);
fun3 = @(block_struct) CompRecSignal(block_struct.data);

%% Find the DCT of the signal
I_dct = blockproc(I,[blocksize,blocksize],fun);


%% Threshold and take random measurements from the signal
I_rec =  blockproc(I_dct,[blocksize,blocksize],fun2);
imshow(uint8(I_rec))
N = (blocksize^2);
T = eye(N);
% Find how sparse the signal is
A = randn([ceil(K*N),N])*T^-1;
A = orth(A')';

I = im2col(I,[blocksize blocksize],'distinct');

y = A*I; %Measurements
xRec = zeros(blocksize^2,NoBlock);

%%%%%%%%For comparison Fowler's Method%%%%%%%
Fy = BCS_SPL_Encoder(I_org,A);
Fowler = BCS_SPL_DCT_Decoder(Fy, A, size(I_org,1), size(I_org,2));
Fwl = col2im(Fowler,[blocksize blocksize],[size(I_org)],'distinct');
imshow(uint8(Fwl))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure
pause(0.1)
NoMeas = size(A,1);
t=clock;

for j = 1:NoMeas*NLoop
    parfor i = 1:NoBlock
        xRec(:,i) =  coSampDec(A,y(:,i),ceil(j*(1/NLoop)),xRec(:,i),'Ent');
    end
    I_Rec =  col2im(xRec,[blocksize blocksize],[size(I_org)],'distinct');
    if j<NoMeas/2
    I_Rec = wiener2(I_Rec,[3,3]);
    else
        if (j<NoMeas*NLoop-2 && rem(j,5)==0&&j>NoMeas/2)||j==NoMeas*NLoop-2
            %display('medfilt');
            I_Rec = medfilt2(I_Rec,[3,3],'symmetric');
        end
    end
%     imshow(uint8(I_Rec));
%     pause(0.1)
    xRec = im2col(I_Rec,[blocksize blocksize],'distinct');
    display([num2str(snr(I_Rec,I_rec)),' vs ',num2str(snr(Fwl,I_rec)),' at iteration ',num2str(j),'/',num2str(NoMeas*NLoop)]);
end

imshow(uint8(I_Rec));title('Image reconstructed'); figure; imshow(uint8(Fwl));title('Fowler Rec');
figure;imshow(uint8(I_org));title('Original');

display(['Percent of the coefs that are kept:',num2str(K*100)]);
display(['SNR between the original and the reconstructed:',num2str(snr(I_Rec,I_rec))]);
display(['SNR between the original and the reconstructed:',num2str(snr(Fwl,I_rec))]);
etime(t,clock)
save([num2str(03),imname,'.mat']);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% CompRecSignal
function [xProj] = coSampDec(A,y,j,xProj,PType)
NoMeas = size(A,1);
if j>NoMeas
    j = length(xProj);
end

%% Start the Iterations
filt = (exp(0:-3/size(A,2):-3+1/size(A,2)));%1;%(1-dct2(kron([0.25,0.5,0.25],[0.25,0.5,0.25]'),length(xProj)))*10;%(exp(-1:-1/size(A,2):-2+1/size(A,2)));%
t = clock;
for i = 1:NoMeas
    switch PType
        case 'L2'
            xProj = L2Proj(A(i,:),y(i),xProj);
        case 'Ent'
            xProj = EntropicProj(A(i,:),y(i),xProj);
    end
end
xProj = dct2(col2im(xProj,[sqrt(size(A,2)),sqrt(size(A,2))],[sqrt(size(A,2)),sqrt(size(A,2))],'distinct'));
[~,indxProj] =sort(abs(xProj(:).*filt'),'descend');
E2 = sum(sum(xProj(indxProj(j+1:end))))*10^-2;
xProj(indxProj(j+1:end)) = xProj(indxProj(j+1:end))*10^-2;
%xProj(indxProj(1:j))+(sign(xProj(indxProj(1:j))).*E2/j);
xProj = idct2(xProj);
xProj = double(uint8(xProj));
xProj = im2col(xProj,[sqrt(size(A,2)),sqrt(size(A,2))],'distinct');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [I] = dctfilter(I,thres)

I = double(I);
I = dct2(I);
I(abs(I)<thres)=0;






























































%